import os

from moviepy.editor import *

def download_url(url, filename):
    if not os.path.exists(filename):
       print("\nDownloading %s\n" % filename)
       download_webfile(url, filename)
       print("Downloading complete...\n")

def download_youtube_video(youtube_id, filename):
    # FYI..  travis-ci doesn't like youtube-dl
    download_url(youtube_id, filename)

def download():
    if not os.path.exists("media"):
       os.mkdir("media")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/images/python_logo.png",
                 "media/python_logo.png")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/images/matplotlib_demo1.png",
                 "media/matplotlib_demo1.png")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/images/afterimage.png",
                 "media/afterimage.png")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/videos/big_buck_bunny_432_433.webm",
                 "media/big_buck_bunny_432_433.webm")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/sounds/crunching.mp3",
                 "media/crunching.mp3")

    download_url("https://raw.githubusercontent.com/earney/moviepy_media/master/tests/subtitles/subtitles1.srt",
                 "media/subtitles1.srt")

    download_url("https://github.com/earney/moviepy_media/raw/master/tests/images/pigs_in_a_polka.gif",
                 "media/pigs_in_a_polka.gif")

    download_url("https://data.vision.ee.ethz.ch/cvl/video2gif/kAKZeIzs0Ag.mp4",
                 "media/video_with_failing_audio.mp4")
    
    download_url("https://github.com/earney/moviepy_media/raw/master/tests/videos/fire2.mp4",
                 "media/fire2.mp4")

    download_url("https://raw.githubusercontent.com/earney/moviepy_media/master/tests/misc/traj.txt",
                 "media/traj.txt")



import pytest
import moviepy.tools as tools
import sys
import time

## taken from PR #121   (grimley517)


def test_ext():
    '''Test for find_extension function'''
    lefts = ['libx264', 'libmpeg4', 'libtheora', 'libvpx']
    rights = ['mp4', 'mp4', 'ogv', 'webm']
    for i in range(len(lefts)):
        left = tools.find_extension(lefts[i])
        right = rights[i]
        message = "{0} did not get associated with {1}".format(left, right)
        assert left == right, message

def test_2():
    '''Tests for raising erre if codec not in dictionaries'''
    message = "asking for a silly video format did not Raise a Value Error"
    with pytest.raises(ValueError, message=message):
         tools.find_extension('flashvideo')

def test_3():
    '''tests the cvsecs funtion outputs the correct times 
       as per the docstring'''
    lefts = [15.4, (1,21.5), (1,1,2), '01:01:33.5', '01:01:33.045' ]
    rights = [15.4, 81.5, 3662, 3693.5, 3693.045]
    for i in range(len(lefts)):
        left = tools.cvsecs(lefts[i])
        right = rights[i]
        message = "{0} resulted in {1}, but {2} was expected"\
            .format(lefts[i],left, right)
        assert left == right, message

def test_4():
    '''tests the is_string function in tools'''
    lefts = ["hello straight string", r'hello raw string',42, True ]
    rights = [True, True, False, False]
    for i in range(len(lefts)):
        left = tools.is_string(lefts[i])
        right = rights[i]
        message = "{0} resulted in {1}, but {2} was expected"\
            .format(lefts[i],left, right)
        assert left == right, message

def test_4a():
    '''as for test 4 - but tests for the different behaviour of byte strings
       between python 2 and 3'''
    version = sys.version_info[0]
    answer = version < 3 #True for py2, else False
    left = tools.is_string(b'hello bytes')
    right = answer
    message = "{0} resulted in {1}, but {2} was expected"\
        .format(b'hello bytes',left, right)
    assert left == right, message

def test_5():
    '''Tests for sys_write-flush function
        1) checks that this works quickly,
        2) checks that stdout has no content after flushing
    '''
    start = time.time()
    tools.sys_write_flush("hello world")
    myTime = time.time() - start
    assert myTime <  0.001
    file = sys.stdout.read()
    assert file == b""

def test_6():
    '''
        Tests subprocess_call for operation.  the process sleep should run for
        a given time in seconds. This checks that the process has 
        deallocated from the stack on completion of the called process
    '''
    process = tools.subprocess_call(["sleep" , '1'])
    time.sleep(1)
    assert process is None

if __name__ == '__main__':
   pytest.main()



"""
Tests meant to be run with pytest
"""

import sys
import os
import pytest

from moviepy.editor import *

from moviepy.video.tools.interpolators import Trajectory

import sys
sys.path.append("tests")
import download_media
from test_helper import PYTHON_VERSION, TMP_DIR, TRAVIS

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_PR_306():
    if TRAVIS:
       return

    #put this back in once we get ImageMagick working on travis-ci
    assert TextClip.list('font') != []
    assert TextClip.list('color') != []

    with pytest.raises(Exception, message="Expecting Exception"):
         TextClip.list('blah')


def test_PR_339():
    if TRAVIS:
       return

    #in caption mode
    overlay = TextClip(txt='foo',
                       color='white', font="Liberation-Mono",
                       size=(640, 480),
                       method='caption',
                       align='center',
                       fontsize=25)

    #in_label_mode
    overlay = TextClip(txt='foo', font="Liberation-Mono", method='label')


def test_PR_373():
    result = Trajectory.load_list("media/traj.txt")

    Trajectory.save_list(result, os.path.join(TMP_DIR, "traj1.txt"))

    result1 = Trajectory.load_list(os.path.join(TMP_DIR,"traj1.txt"))

    assert len(result[0].tt) == len(result1[0].tt)
    for i in range(len(result[0].tt)):
        assert result[0].tt[i] == result1[0].tt[i]

    assert len(result[0].xx) == len(result1[0].xx)
    for i in range(len(result[0].xx)):
        assert result[0].xx[i] == result1[0].xx[i]

    assert len(result[0].yy) == len(result1[0].yy)
    for i in range(len(result[0].yy)):
        assert result[0].yy[i] == result1[0].yy[i]


def test_PR_424():
    # Recommended use
    clip = ColorClip([1000, 600], color=(60, 60, 60), duration=10)
    # Uses `col` so should work the same as above, but give warning
    clip = ColorClip([1000, 600], col=(60, 60, 60), duration=10)
    # Should give 2 warnings and use `color`, not `col`
    clip = ColorClip([1000, 600], color=(60, 60, 60), duration=10, col=(2,2,2))


def test_PR_458():
    clip = ColorClip([1000, 600], color=(60, 60, 60), duration=10)
    clip.write_videofile(os.path.join(TMP_DIR, "test.mp4"),
                         progress_bar=False, fps=30)


def test_PR_515():
    # Won't actually work until video is in download_media
    clip = VideoFileClip("media/fire2.mp4", fps_source='tbr')
    assert clip.fps == 90000
    clip = VideoFileClip("media/fire2.mp4", fps_source='fps')
    assert clip.fps == 10.51

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *

def test_clips_array():
    red = ColorClip((1024,800), color=(255,0,0))
    green = ColorClip((1024,800), color=(0,255,0))
    blue = ColorClip((1024,800), color=(0,0,255))

    video = clips_array([[red, green, blue]])

    with pytest.raises(ValueError,
                       message="Expecting ValueError (duration not set)"):
       video.resize(width=480).write_videofile("/tmp/test_clips_array.mp4")


def test_clips_array():
    red = ColorClip((1024,800), color=(255,0,0))
    green = ColorClip((1024,800), color=(0,255,0))
    blue = ColorClip((1024,800), color=(0,0,255))

    video = clips_array([[red, green, blue]]).set_duration(5)

    with pytest.raises(AttributeError,
                       message="Expecting ValueError (fps not set)"):
         video.write_videofile("/tmp/test_clips_array.mp4")


    #this one should work correctly
    red.fps=green.fps=blue.fps=30
    video = clips_array([[red, green, blue]]).set_duration(5)
    video.write_videofile("/tmp/test_clips_array.mp4")


if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *
import os

def test_setup():
    red = ColorClip((1024,800), color=(255,0,0))
    green = ColorClip((1024,800), color=(0,255,0))
    blue = ColorClip((1024,800), color=(0,0,255))

    red.fps=green.fps=blue.fps=30
    video = clips_array([[red, green, blue]]).set_duration(5)
    video.write_videofile("/tmp/test.mp4")

    assert os.path.exists("/tmp/test.mp4")

    clip = VideoFileClip("/tmp/test.mp4")
    assert clip.duration == 5
    assert clip.fps == 30
    assert clip.size == [1024*3, 800]

def test_ffmpeg_resizing():
    # Test downscaling
    target_resolution = (128,128)
    video = VideoFileClip("media/big_buck_bunny_432_433.webm", target_resolution=target_resolution)
    frame = video.get_frame(0)
    assert frame.shape[0:2] == target_resolution

    target_resolution = (128, None)
    video = VideoFileClip("media/big_buck_bunny_432_433.webm", target_resolution=target_resolution)
    frame = video.get_frame(0)
    assert frame.shape[0] == target_resolution[0]

    target_resolution = (None, 128)
    video = VideoFileClip("media/big_buck_bunny_432_433.webm", target_resolution=target_resolution)
    frame = video.get_frame(0)
    assert frame.shape[1] == target_resolution[1]

    # Test upscaling
    target_resolution = (None, 2048)
    video = VideoFileClip("media/big_buck_bunny_432_433.webm", target_resolution=target_resolution)
    frame = video.get_frame(0)
    assert frame.shape[1] == target_resolution[1]


if __name__ == '__main__':
   pytest.main()



"""
To run the tests:
    py.test tests.py
"""

from moviepy.editor import *

#@pytest.fixture
def test_if_TextClip_crashes_in_caption_mode():
    overlay = TextClip(txt='foo',
                       color='white',
                       size=(640, 480),
                       method='caption',
                       align='center',
                       fontsize=25)

def test_if_TextClip_crashes_in_label_mode():
    overlay = TextClip(txt='foo', method='label')



import pytest
from moviepy.editor import *
import moviepy.video.tools.cuts as cuts

import os
import sys
sys.path.append("tests")
import download_media
from test_helper import TRAVIS, TMP_DIR


def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_cuts1():
    clip = VideoFileClip("media/big_buck_bunny_432_433.webm").resize(0.2)
    cuts.find_video_period(clip) == pytest.approx(0.966666666667, 0.0001)

def test_subtitles():
    from moviepy.video.tools.subtitles import SubtitlesClip

    red = ColorClip((800, 600), color=(255,0,0)).set_duration(10)
    green = ColorClip((800, 600), color=(0,255,0)).set_duration(10)
    blue = ColorClip((800, 600), color=(0,0,255)).set_duration(10)
    myvideo = concatenate_videoclips([red,green,blue])
    assert myvideo.duration == 30

    #travis does not like TextClip.. so return for now..
    #but allow regular users to still run the test below
    if TRAVIS:
       return

    generator = lambda txt: TextClip(txt, font='Liberation-Mono',
                                     size=(800,600), fontsize=24,
                                     method='caption', align='South',
                                     color='white')

    subtitles = SubtitlesClip("media/subtitles1.srt", generator)
    final = CompositeVideoClip([myvideo, subtitles])
    final.to_videofile(os.path.join(TMP_DIR, "subtitles1.mp4"), fps=30)

    data = [([0.0, 4.0], 'Red!'), ([5.0, 9.0], 'More Red!'),
            ([10.0, 14.0], 'Green!'), ([15.0, 19.0], 'More Green!'),
            ([20.0, 24.0], 'Blue'), ([25.0, 29.0], 'More Blue!')]

    assert subtitles.subtitles == data

    subtitles = SubtitlesClip(data, generator)
    assert subtitles.subtitles == data

def test_file_to_subtitles():
    from moviepy.video.tools.subtitles import file_to_subtitles

    data = [([0.0, 4.0], 'Red!'), ([5.0, 9.0], 'More Red!'),
            ([10.0, 14.0], 'Green!'), ([15.0, 19.0], 'More Green!'),
            ([20.0, 24.0], 'Blue'), ([25.0, 29.0], 'More Blue!')]

    assert data == file_to_subtitles("media/subtitles1.srt")

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *

# must have to work on travis-ci
import sys
sys.path.append("tests")
import download_media

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_afterimage():
    ai=ImageClip("media/afterimage.png")
    masked_clip = vfx.mask_color(ai, color=[0,255,1]) # for green

    some_background_clip = ColorClip((800,600), color=(255,255,255))

    final_clip = CompositeVideoClip([some_background_clip, masked_clip],
                                    use_bgclip=True)
    final_clip.duration=5
    final_clip.write_videofile("/tmp/afterimage.mp4", fps=30)

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *

# must have to work on travis-ci
import sys
sys.path.append("tests")
import download_media

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_1():
    images=[]
    durations=[]
    for i in range(5):
        durations.append(i)
        images.append("media/python_logo.png")
        durations.append(i)
        images.append("media/matplotlib_demo1.png")

    clip=ImageSequenceClip(images, durations=durations)
    assert clip.duration == sum(durations)
    clip.write_videofile("/tmp/ImageSequenceClip1.mp4", fps=30)

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *
import moviepy.video.tools.cuts as cuts

import os
import sys
sys.path.append("tests")
import download_media
from test_helper import PYTHON_VERSION, TMP_DIR, TRAVIS

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_matplotlib():
    if PYTHON_VERSION in ('2.7', '3.3'):
       return

    #for now, python 3.5 installs a version of matplotlib that complains
    #about $DISPLAY variable, so lets just ignore for now.
    if PYTHON_VERSION == '3.5' and TRAVIS:
       return

    import matplotlib.pyplot as plt
    import numpy as np
    from moviepy.editor import VideoClip
    from moviepy.video.io.bindings import mplfig_to_npimage

    x = np.linspace(-2, 2, 200)

    duration = 2

    fig, ax = plt.subplots()

    def make_frame(t):
        ax.clear()
        ax.plot(x, np.sinc(x**2) + np.sin(x + 2*np.pi/duration * t), lw=3)
        ax.set_ylim(-1.5, 2.5)
        return mplfig_to_npimage(fig)

    animation = VideoClip(make_frame, duration=duration)
    animation.write_gif(os.path.join(TMP_DIR, 'matplotlib.gif'), fps=20)

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *

import sys
sys.path.append("tests")
import download_media

def test_duration():
    return
    #TextClip returns the following error under Travis (issue with Imagemagick)
    #convert.im6: not authorized `@/tmp/tmpWL7I3M.txt' @ error/property.c/InterpretImageProperties/3057.
    #convert.im6: no images defined `PNG32:/tmp/tmpRZVqGQ.png' @ error/convert.c/ConvertImageCommand/3044.

    clip = TextClip('hello world', size=(1280,720), color='white')
    clip.set_duration(5)
    assert clip.duration == 5

    clip2 = clip.fx(vfx.blink, d_on=1, d_off=1)
    clip2.set_duration(5)
    assert clip2.duration == 5

if __name__ == '__main__':
   pytest.main()



import pytest
from moviepy.editor import *

from moviepy.video.io.ffmpeg_reader import ffmpeg_parse_infos

import sys
sys.path.append("tests")
import download_media

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_ffmpeg_parse_infos():
    d=ffmpeg_parse_infos("media/big_buck_bunny_432_433.webm")
    assert d['duration'] == 1.0

    d=ffmpeg_parse_infos("media/pigs_in_a_polka.gif")
    assert d['video_size'] == [314, 273]
    assert d['duration'] == 3.0


if __name__ == '__main__':
   pytest.main()



"""
Tests meant to be run with pytest
"""

import os
import pytest

from moviepy.editor import *

import sys
sys.path.append("tests")
import download_media
from test_helper import PYTHON_VERSION, TMP_DIR, TRAVIS

def test_download_media(capsys):
    with capsys.disabled():
       download_media.download()

def test_issue_145():
    video = ColorClip((800, 600), color=(255,0,0)).set_duration(5)
    with pytest.raises(Exception, message="Expecting Exception"):
         final = concatenate_videoclips([video], method = 'composite')

def test_issue_285():
    clip_1 = ImageClip('media/python_logo.png', duration=10)
    clip_2 = ImageClip('media/python_logo.png', duration=10)
    clip_3 = ImageClip('media/python_logo.png', duration=10)

    merged_clip = concatenate_videoclips([clip_1, clip_2, clip_3])
    assert merged_clip.duration == 30

def test_issue_354():
    clip = ImageClip("media/python_logo.png")

    clip.duration = 10
    crosstime = 1

    #caption = editor.TextClip("test text", font="Liberation-Sans-Bold", color='white', stroke_color='gray', stroke_width=2, method='caption', size=(1280, 720), fontsize=60, align='South-East')
    #caption.duration = clip.duration
    fadecaption = clip.crossfadein(crosstime).crossfadeout(crosstime)
    ret = CompositeVideoClip([clip, fadecaption])

def test_issue_359():
    video = ColorClip((800, 600), color=(255,0,0)).set_duration(5)
    video.fps=30
    video.write_gif(filename=os.path.join(TMP_DIR, "issue_359.gif"),
                    tempfiles=True)

def test_issue_368():
    import sys
    if PYTHON_VERSION in ('2.7', '3.3'): #matplotlib only supported in python >= 3.4
       return

    #travis, python 3.5 matplotlib version has problems..
    if PYTHON_VERSION == '3.5' and TRAVIS:
       return

    import numpy as np
    import matplotlib.pyplot as plt
    from sklearn import svm
    from sklearn.datasets import make_moons
    from moviepy.video.io.bindings import mplfig_to_npimage
    import imageio

    X, Y = make_moons(50, noise=0.1, random_state=2) # semi-random data

    fig, ax = plt.subplots(1, figsize=(4, 4), facecolor=(1,1,1))
    fig.subplots_adjust(left=0, right=1, bottom=0)
    xx, yy = np.meshgrid(np.linspace(-2,3,500), np.linspace(-1,2,500))

    def make_frame(t):
        ax.clear()
        ax.axis('off')
        ax.set_title("SVC classification", fontsize=16)

        classifier = svm.SVC(gamma=2, C=1)
        # the varying weights make the points appear one after the other
        weights = np.minimum(1, np.maximum(0, t**2+10-np.arange(50)))
        classifier.fit(X, Y, sample_weight=weights)
        Z = classifier.decision_function(np.c_[xx.ravel(), yy.ravel()])
        Z = Z.reshape(xx.shape)
        ax.contourf(xx, yy, Z, cmap=plt.cm.bone, alpha=0.8,
                    vmin=-2.5, vmax=2.5, levels=np.linspace(-2,2,20))
        ax.scatter(X[:,0], X[:,1], c=Y, s=50*weights, cmap=plt.cm.bone)

        return mplfig_to_npimage(fig)

    animation = VideoClip(make_frame,duration=2)
    animation.write_gif(os.path.join(TMP_DIR, "svm.gif"),fps=20)

def test_issue_407():
    red = ColorClip((800, 600), color=(255,0,0)).set_duration(5)
    red.fps=30
    assert red.fps == 30
    assert red.w == 800
    assert red.h == 600
    assert red.size == (800, 600)

    #ColorClip has no fps attribute
    green=ColorClip((640, 480), color=(0,255,0)).set_duration(2)
    blue=ColorClip((640, 480), color=(0,0,255)).set_duration(2)

    assert green.w == blue.w == 640
    assert green.h == blue.h == 480
    assert green.size == blue.size == (640, 480)

    with pytest.raises(AttributeError, message="Expecting ValueError Exception"):
         green.fps

    with pytest.raises(AttributeError, message="Expecting ValueError Exception"):
         blue.fps

    video=concatenate_videoclips([red, green, blue])
    assert video.fps == red.fps

def test_issue_416():
    green=ColorClip((640, 480), color=(0,255,0)).set_duration(2)  #ColorClip has no fps attribute
    video1=concatenate_videoclips([green])
    assert video1.fps == None

def test_issue_417():
    # failed in python2

    cad = u'media/python_logo.png'
    myclip = ImageClip(cad).fx(vfx.resize, newsize=[1280, 660])
    final = CompositeVideoClip([myclip], size=(1280, 720))
    #final.set_duration(7).write_videofile("test.mp4", fps=30)

def test_issue_467():
    cad = 'media/python_logo.png'
    clip = ImageClip(cad)

    #caused an error, NameError: global name 'copy' is not defined
    clip = clip.fx(vfx.blink, d_on=1, d_off=1)

def test_issue_470():
    audio_clip = AudioFileClip('media/crunching.mp3')

    # t_end is out of bounds
    subclip = audio_clip.subclip(t_start=6, t_end=9)

    with pytest.raises(IOError, message="Expecting IOError"):
         subclip.write_audiofile('/tmp/issue_470.wav', write_logfile=True)

    #but this one should work..
    subclip = audio_clip.subclip(t_start=6, t_end=8)
    subclip.write_audiofile(os.path.join(TMP_DIR, 'issue_470.wav'), write_logfile=True)

def test_issue_246():
    def test_audio_reader():
        video = VideoFileClip('media/video_with_failing_audio.mp4')
        subclip = video.subclip(270)
        subclip.write_audiofile(os.path.join(TMP_DIR, 'issue_246.wav'),
                                write_logfile=True)

if __name__ == '__main__':
   pytest.main()
